## $Id: test_contract.py 12697 2015-03-09 01:51:12Z uli $
##
## Copyright (C) 2014 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""
Tests for contracts.
"""
import decimal
from zope.interface.verify import verifyClass, verifyObject
from zope.component import createObject
from hurry.workflow.interfaces import (
    IWorkflowInfo, IWorkflowState, InvalidTransitionError)
from waeup.ikoba.customers.interfaces import (
    IContractsContainer, IContract)
from waeup.ikoba.customers.contracts import (
    ContractsContainer, SampleContract, payment_items_from_contract,
    ContractPayer,
    )
from waeup.ikoba.customers.customer import Customer
from waeup.ikoba.payments.interfaces import IPaymentItem, IPayer
from waeup.ikoba.products.productoptions import ProductOption
from waeup.ikoba.testing import (FunctionalLayer, FunctionalTestCase)


class ContractsContainerTestCase(FunctionalTestCase):

    layer = FunctionalLayer

    def test_interfaces(self):
        # Make sure the correct interfaces are implemented.
        self.assertTrue(
            verifyClass(
                IContractsContainer, ContractsContainer)
            )
        self.assertTrue(
            verifyObject(
                IContractsContainer, ContractsContainer())
            )
        self.assertTrue(
            verifyClass(
                IContract, SampleContract)
            )
        self.assertTrue(
            verifyObject(
                IContract, SampleContract())
            )
        return

    def test_addContract(self):
        container = ContractsContainer()
        contract = createObject(u'waeup.SampleContract')
        id = contract.contract_id
        container.addContract(contract)
        self.assertEqual(container[id], contract)
        self.assertRaises(TypeError, container.addContract, object())
        self.assertEqual(contract.class_name, 'SampleContract')
        return

    def test_contract_workflow(self):
        contract = createObject(u'waeup.SampleContract')
        IWorkflowInfo(contract).fireTransition('create')
        self.assertEqual(IWorkflowState(contract).getState(), 'created')
        IWorkflowInfo(contract).fireTransition('submit')
        self.assertEqual(IWorkflowState(contract).getState(), 'submitted')
        IWorkflowInfo(contract).fireTransition('approve')
        self.assertEqual(IWorkflowState(contract).getState(), 'approved')
        IWorkflowInfo(contract).fireTransition('expire')
        self.assertEqual(IWorkflowState(contract).getState(), 'expired')
        IWorkflowInfo(contract).fireTransition('reset4')
        self.assertEqual(IWorkflowState(contract).getState(), 'created')
        self.assertRaises(InvalidTransitionError,
            IWorkflowInfo(contract).fireTransition, 'approve')
        IWorkflowInfo(contract).fireTransition('submit')
        IWorkflowInfo(contract).fireTransition('reset3')
        self.assertEqual(IWorkflowState(contract).getState(), 'created')
        return

    def test_contract_history(self):
        doc = createObject(u'waeup.SampleContract')
        IWorkflowInfo(doc).fireTransition('create')
        messages = ' '.join(doc.history.messages)
        self.assertTrue('Contract created by system' in messages)
        return


class TestContractHelpers(FunctionalTestCase):

    layer = FunctionalLayer

    def test_payment_items_from_contract(self):
        # we can turn contracts into lists of payment items
        contract = SampleContract()
        option1 = ProductOption(u"Fee 1", decimal.Decimal("31.10"), "USD")
        option2 = ProductOption(u"Fee 2", decimal.Decimal("12.12"), "USD")
        contract.product_options = [option1, option2]
        payment_items = payment_items_from_contract(contract)
        assert len(payment_items) == 2
        item1, item2 = payment_items
        assert IPaymentItem.providedBy(item1)
        assert IPaymentItem.providedBy(item2)
        assert item1.item_id == "0"
        assert item2.item_id == "1"
        assert item1.title == u'Fee 1'
        assert item2.title == u'Fee 2'
        assert item1.amount == decimal.Decimal("31.10")
        assert item2.amount == decimal.Decimal("12.12")

    def test_payment_item_from_contract_no_options(self):
        # we cope with empty contracts
        contract = SampleContract()
        payment_items = payment_items_from_contract(contract)
        assert payment_items == []

    def test_payer_adapter(self):
        # we can adapt IContract to IPayer (i.e. create a payer)
        customer = Customer()
        customer.firstname, customer.lastname = u'Anna', u'Tester'
        contract = createObject(u'waeup.SampleContract')
        customer['contracts'] = ContractsContainer()
        customer['contracts'].addContract(contract)
        result = IPayer(contract)
        self.assertTrue(isinstance(result, ContractPayer))
        verifyObject(IPayer, result)
        self.assertEqual(result.first_name, u'Anna')
        self.assertEqual(result.last_name, u'Tester')
        self.assertEqual(result.payer_id, customer.customer_id)
